/*************************************************************************/
/* Headerfile fr AMD-FLASH - Treiber miniMODUL-166                      */
/*                                                                       */
/* Version 1.0 - 06.03.94 /Kl                                            */
/* Version 1.1 - 25.11.94 /Hu                                            */
/*                                                                       */
/* Revisions:                                                            */
/*                                                                       */
/* (c) 1994 PHYTEC Messtechnik GmbH Mainz                                */
/*************************************************************************/

#ifndef _AMD_FLAS_H_
#define _AMD_FLAS_H_

#ifndef DWORD
    #define DWORD unsigned long
#endif
#ifndef WORD
    #define WORD unsigned int
#endif
#ifndef BYTE
    #define BYTE unsigned char
#endif



#define AMD_HARDWARE_PROTECTED   0x01
#define AMD_SOFTWARE_PROTECTED   0x02
#define AMD_PARTIAL_ACCESS       0x04
#define AMD_NO_ACCESS            0x0C
#define AMD_BLANK                0x10

#define AMD_SUCCESSFUL           0x00
#define AMD_CONFIG_ERROR         0xF6
#define AMD_INVALID_DEVICE       0xF7
#define AMD_LOCATION_NOT_EMPTY   0xF8
#define AMD_PROG_TIMEOUT         0xF9
#define AMD_ADDR_NO_ACCESS       0xFA
#define AMD_ADDR_PROTECTED       0xFB
#define AMD_ADDR_OUT_OF_RANGE    0xFC
#define AMD_ADDR_INVALID         0xFD
#define AMD_ERASE_TIMEOUT        0xFE
#define AMD_INVALID_SECTOR       0xFF



#define EVEN                     0
#define ODD                      1



extern BYTE AMDAddr2Sectors(DWORD StartAddr, DWORD EndAddr,
                             BYTE *StartSec, BYTE *EndSec);
/****************************************************************************/
/* FUNCTION AMDAddr2Sectors                                                 */
/* Berechnen der zum Speicherbereich gehrenden Sektornummern               */
/*                                                                          */
/* benutzte Funktionen: keine                                               */
/*                                                                          */
/* bergabeparameter  : Startadresse, Endadresse                            */
/*                                                                          */
/* Rckgabewert: AMD_SUCCESSFUL bei korrekter Berechnung,                   */
/*               AMD_ADDR_OUT_OF_RANGE oder AMD_ADDR_INVALID sonst.         */
/*                                                                          */
/* Rckgabe der betroffene Sektoren: erster in *StartSec, letzter in *EndSec*/
/*                                                                          */
/* !!! KEINE BERCKSICHTIGUNG DES GESCHTZTEN BEREICHS !!!                  */
/*                                                                          */
/****************************************************************************/

extern BYTE AMDEraseChip(BYTE Odd);    /* Lschen eines gesamten Chips      */
/****************************************************************************/
/* FUNCTION AMDEraseChip                                                    */
/* Komplettes Lschen des Flash-Speichers                                   */
/*                                                                          */
/* benutzte Funktionen: keine                                               */
/*                                                                          */
/* bergabeparameter  : Flag fr Even/Odd-Speicherselektion                 */
/*                                                                          */
/* Rckgabewert: AMD_SUCCESSFUL oder AMD_ERASE_TIMEOUT                      */
/****************************************************************************/

extern BYTE AMDEraseSector(BYTE Sec, BYTE Odd);
/****************************************************************************/
/* FUNCTION AMDEraseSector                                                  */
/* Lschen eines Flash-Sektors                                              */
/*                                                                          */
/* benutzte Funktionen: AMDGetSectorAddr, AMDGetSectorStatus                */
/*                                                                          */
/* bergabeparameter  : Sektornummer, Even-/Odd-Flag                        */
/*                                                                          */
/* Rckgabewert: AMD_SUCCESSFUL, AMD_INVALID_SECTOR, AMD_ERASE_TIMEOUT,     */
/*               AMD_CONFIG_ERROR oder                                      */
/*               Bitkombination aus AMD_HARDWARE_PROTECTED,                 */
/*               AMD_SOFTWARE_PROTECTED, AMD_PARTIAL_ACCESS und             */
/*               AMD_NO_ACCESS                                              */
/*                                                                          */
/****************************************************************************/

extern WORD AMDGetChipCodes(BYTE Odd);
/****************************************************************************/
/* FUNCTION AMDGetChipCodes                                                 */
/* Liest Hersteller und Typ-Kodes des Flash-Chips                           */
/*                                                                          */
/* benutzte Funktionen: keine                                               */
/*                                                                          */
/* bergabeparameter  : Flag fr Even/Odd-Speicherselektion                 */
/*                                                                          */
/* Rckgabewert: Typ-Kode (MSB=Manufacturer Code, LSB=Device Code)          */
/****************************************************************************/

extern char *AMDGetChipName(BYTE Odd);
/****************************************************************************/
/* FUNCTION AMDGetChipName                                                  */
/* Ergibt Bezeichnung des Flash-Chips                                       */
/*                                                                          */
/* benutzte Funktionen: AMDGetChipCodes                                     */
/*                                                                          */
/* bergabeparameter  : Flag fr Even/Odd-Speicherselektion                 */
/*                                                                          */
/* Rckgabewert: String mit Namen, NULL wenn nicht bekannt                  */
/****************************************************************************/

extern BYTE AMDGetSectorAddr(BYTE Sec, BYTE Odd, DWORD *StartAddr,
                             DWORD *EndAddr);
/****************************************************************************/
/* FUNCTION AMDGetSectorAddr                                                */
/* Berechnen der Speicheradressen eines Sektors                             */
/*                                                                          */
/* benutzte Funktionen: keine                                               */
/*                                                                          */
/* bergabeparameter  : Sektornummer, Even-/Odd-Flag                        */
/*                                                                          */
/* Rckgabewert: AMD_SUCCESSFUL, AMD_CONFIG_ERROR oder                      */
/*               AMD_INVALID_SECTOR                                         */
/*                                                                          */
/* Rckgabewerte ber Pointer: Adressbereich des Sektors (Start und Ende)   */
/*                                                                          */
/****************************************************************************/

extern BYTE AMDGetSectorStatus(BYTE Sec, BYTE Odd);
/****************************************************************************/
/* FUNCTION AMDGetSectorStatus                                              */
/* berprft auf Vorhandensein einer Sektorsperre (protection)              */
/*                                                                          */
/* benutzte Funktionen: AMDGetSectorAddr                                    */
/*                                                                          */
/* bergabeparameter  : Sektornummer, Even-/Odd-Flag                        */
/*                                                                          */
/* Rckgabewert: Statusflag, i.e.                                           */
/*               AMD_INVALID_SECTOR, AMD_CONFIG_ERROR oder eine Bit-        */
/*               kombination aus                                            */
/*               AMD_HARDWARE_PROTECTED, AMD_SOFTWARE_PROTECTED,            */
/*               AMD_FULL_ACCESS, AMD_PARTIAL_ACCESS, AMD_NO_ACCESS,        */
/*               AMD_BLANK                                                  */
/****************************************************************************/

extern BYTE AMDInitDriver(void);
/****************************************************************************/
/* FUNCTION AMDInitDriver                                                   */
/* Initialisiert den Flash-Treiber                                          */
/*                                                                          */
/* benutzte Funktionen: AMDGetChipCodes                                     */
/*                                                                          */
/* bergabeparameter  : keine                                               */
/*                                                                          */
/* Rckgabewert: AMD_SUCCESSFUL, AMD_CONFIG_ERROR oder AMD_INVALID_DEVICE   */
/****************************************************************************/

extern BYTE AMDProgByte(DWORD Addr, BYTE b);
/****************************************************************************/
/* FUNCTION AMDProgBYTE                                                     */
/* Programmierung eines Bytes in AMD-Flash.                                 */
/*                                                                          */
/* benutzte Funktionen: keine                                               */
/*                                                                          */
/* bergabeparameter  : Adresse und Datum                                   */
/*                                                                          */
/* Rckgabewert: AMD_SUCCESSFUL bei Erfolg,                                 */
/*               AMD_PROG_TIMEOUT, wenn interner Timeout                    */
/*               AMD_ADDR_NO_ACCESS, wenn Adresse innerhalb eines Bereichs  */
/*                                   ohne Zugriff                           */
/*               AMD_ADDR_OUT_OF_RANGE, wenn Adresse auerhalb des          */
/*                                      Speicherbereichs                    */
/*               AMD_ADDR_PROTECTED, wenn Adresse innerhalb eines           */
/*                                   geschtzten Bereichs                   */
/*               AMD_LOCATION_NOT_EMPTY, wenn Speicherzelle nicht leer      */
/****************************************************************************/

extern BYTE  AMDSectorBlankCheck(BYTE Sec, BYTE Odd);
/****************************************************************************/
/* FUNCTION AMDSectorBlankCheck                                             */
/* fhrt einen Blank-Check fr einen Sektor durch                           */
/*                                                                          */
/* benutzte Funktionen: AMDGetSectorAddr, AMDGetSectorStatus                */
/*                                                                          */
/* bergabeparameter  : Sektornummer, Even-/Odd-Flag                        */
/*                                                                          */
/* Rckgabewert: Statusflag, i.e.                                           */
/*               AMD_INVALID_SECTOR, AMD_CONFIG_ERROR oder eine Bit-        */
/*               kombination aus                                            */
/*               AMD_HARDWARE_PROTECTED, AMD_SOFTWARE_PROTECTED,            */
/*               AMD_FULL_ACCESS, AMD_PARTIAL_ACCESS, AMD_NO_ACCESS,        */
/*               AMD_BLANK                                                  */
/****************************************************************************/



#ifdef NO_INIT                         /* ohne Initialisierung im Sonderfall*/

DWORD FlashStart;                      /* physikalischer Start des Flash    */
DWORD FlashEnd;                        /* physikalisches Ende des Flash     */
DWORD Protect1Start;                   /* Start erster geschtzter Bereich  */
DWORD Protect1End;                     /* Ende erster geschtzter Bereich   */
DWORD Protect2Start;                   /* Start zweiter geschtzter Bereich */
DWORD Protect2End;                     /* Ende zweiter geschtzter Bereich  */
DWORD Protect3Start;                   /* Start dritter geschtzter Bereich */
DWORD Protect3End;                     /* Ende dritter geschtzter Bereich  */
DWORD NoAccess1Start;                  /* Start erster Bereich ohne Zugriff */
DWORD NoAccess1End;                    /* Ende erster Bereich ohne Zugriff  */
DWORD NoAccess2Start;                  /* Start zweiter Bereich ohne Zugriff*/
DWORD NoAccess2End;                    /* Ende zweiter Bereich ohne Zugriff */
DWORD NoAccess3Start;                  /* Start dritter Bereich ohne Zugriff*/
DWORD NoAccess3End;                    /* Ende dritter Bereich ohne Zugriff */

#else                                  /* normalerweise mit Initialisierung */

DWORD FlashStart     = 0x00000L;       /* physikalischer Start des Flash    */
DWORD FlashEnd       = 0x1FFFFL;       /* physikalisches Ende des Flash     */
DWORD Protect1Start  = 0x00000L;       /* Start erster geschtzter Bereich  */
DWORD Protect1End    = 0x00000L;       /* Ende erster geschtzter Bereich   */
DWORD Protect2Start  = 0x00000L;       /* Start zweiter geschtzter Bereich */
DWORD Protect2End    = 0x00000L;       /* Ende zweiter geschtzter Bereich  */
DWORD Protect3Start  = 0x00000L;       /* Start dritter geschtzter Bereich */
DWORD Protect3End    = 0x00000L;       /* Ende dritter geschtzter Bereich  */
DWORD NoAccess1Start = 0x0FA00L;       /* Start erster Bereich ohne Zugriff */
DWORD NoAccess1End   = 0x0FFFFL;       /* Ende erster Bereich ohne Zugriff  */
DWORD NoAccess2Start = 0x00000L;       /* Start zweiter Bereich ohne Zugriff*/
DWORD NoAccess2End   = 0x00000L;       /* Ende zweiter Bereich ohne Zugriff */
DWORD NoAccess3Start = 0x00000L;       /* Start dritter Bereich ohne Zugriff*/
DWORD NoAccess3End   = 0x00000L;       /* Ende dritter Bereich ohne Zugriff */

#endif

#endif
