/*****************************************************************************
 ASC0_3.c
                                                                    
   The ASC0 receive to a buffer and transmit from a buffer 20 Byte each 
   (8 Bit, no Parity) by help from interrupt service routine.
   The asynchronous serial interface work in mode 1 Baudrate: 9600 Baud.
   S0RIR release 20 ISR and write all receive chars to buffer s0_buffer.
   After thet stop the receive and enable a transmit interrupt for new
   transmit.        
   S0TBIR release 20 ISR, read all chars from buffer s0_buffer and transmit   
   it over serial interface. After thet stop trensmit and initialize interrupt
   control register SORIC for new receive.      
   
   S0TBIC: ILVL = 15, GLVL = 2, S0TBIE = 1                              
   S0RIC:  ILVL = 15, GLVL = 3, S0RIE = 1                               
   The three Error Interrupts is not enable. 
            
												 PHYTEC Messtechnik GmbH, 2000
 ******************************************************************************/
/* ----- Include Dateien: --------------------------------------------------- */
#include <REG167.H>          		  /* Register Definition for C167         */
#include <INTRINS.H>         		  /* spezific device Funktionen			  */

/* ----- C-globale Byte-/Wort-Objekte: -------------------------------------- */
#define BUFFERVAL 19				  /* value of data buffer 19 + 1 = 20     */
unsigned char write_counter = 0, read_counter = 0;
unsigned int idata s0_buffer[BUFFERVAL];/* insigned 16 Bit C-Array            */

/* ----- S0Receive-Interrupt program ----------------------------------------- */
void s0rint (void) interrupt 0x2b
{  
   s0_buffer[write_counter] = S0RBUF; /* write S0RBUF in to buffer			  */
   if (write_counter < BUFFERVAL) write_counter++; /* increment write counter */
   else								  
   {
     S0RIC = 0;						  /* stop receive						  */
	 write_counter = 0;				 	
     S0TBIC = 0xFE;                   /*init serial intervace transmit interrupt*/
									  /* for transmit						  */		   
   }
}

/* ----- S0Transmit-Interrupt program: -------------------------------------- */
void s0tint (void) interrupt 0x47
{
   S0TBUF = s0_buffer[read_counter];  /* read buffer in to S0TBUF             */
   if (read_counter < BUFFERVAL) read_counter++; /* increment write counter   */
   else 
   {
     S0TBIC = 0;					  /* stop transmit						  */
	 read_counter = 0;
	 S0RIC = 0x7F;					  /*init serial intervace receive interrupt*/
    								  /* for new receive						  */
   }	
}

/* ----- main program: ------------------------------------------------------ */
void main ( )
{  
   P3  |= 0x0400;             		   /* set port 3.10 output latch (TXD)    */
   DP3 |= 0x0400;             		   /* configure port 3.10 for output      */
                              		   /* operation. ( TXD output)            */
   DP3 &= 0xF7FF;             		   /* configure port 3.11 for input       */
                              		   /* operation. ( RXD input)             */

   IEN   = 1;                 		   /* globale Interrupt-Freigabe          */

   S0RIC  = 0x7F;
   S0TBIC = 0x7E;             		   /* init S0TBuffer-Interrupt for PEC 6  */
   
   S0BG  = 0x40;              		   /* load Baudrate generator  9600 baud  */	
   S0CON = 0x8011;            		   /* init seral interface 0              */


   while(1)                   		   /* endless loop                        */
   {
    /* here your program code   											  */
   } 
} /* ----- Ende: ------------------------------------------------------------ */
