/******************************************************************************
 ASC0_lib.c

 This function initialize the Asynchronous/Synchronous Serial
 Interface 0 in Asynchronous Mode with 8-bit data, one Stop bit, no 
 Parity and variable Baudrate.This function supported Baudrate from 50, 75, 110,
 300, 600, 1200, 2400, 4800, 9600, 19200, 38400, 57600 and 625000 baud.


												 PHYTEC Messtechnik GmbH, 1999
*******************************************************************************/
#include <REG167.H>                    /* Define register for C167            */
#include <INTRINS.H>                   /* Prototyping derivate functionen     */

/* ----- Code: serial() funcrion: ------------------------------------------- */
extern void serial (unsigned long BaudRate )
{
/* init serial0 port: Asynchronous Mode 8-bit Data, one Stop bit, no Parity   */  
  P3  |= 0x0400;                       /* set port 3.10 output latch (TXD)    */
  DP3 |= 0x0400;                       /* configure port 3.10 for output      */
                                       /* operation. ( TXD output)            */
  DP3 &= 0xF7FF;                       /* configure port 3.11 for input       */
                                       /* operation. ( RXD input)             */
  S0TIC  = 0x80;                       /* set transmit interrupt flag         */
  S0RIC  = 0x00;                       /* delete receive interrupt flag       */
  S0CON  = 0x0011;                     /* set serial mode                     */

  switch (BaudRate)
  {
   case     50:  S0BG   = 0x1FFF;      /* set baudrate to   50 baud           */
                 S0CON |= 0x2000;	   // Baud Rate Generator frequency = fCPU/3
			     break;
   case     75:  S0BG   = 0x1FFF; break;   /* set baudrate to     75 baud     */
   case    110:  S0BG   = 0x1630; break;   /* set baudrate to    110 baud     */
   case    300:  S0BG   = 0x0822; break;   /* set baudrate to    300 baud     */
   case    600:  S0BG   = 0x0410; break;   /* set baudrate to    600 baud     */
   case   1200:  S0BG   = 0x0207; break;   /* set baudrate to   1200 baud     */
   case   2400:  S0BG   = 0x0103; break;   /* set baudrate to   2400 baud     */
   case   4800:  S0BG   = 0x0081; break;   /* set baudrate to   4800 baud     */
   case   9600:  S0BG   = 0x0040; break;   /* set baudrate to   9600 baud     */
   case  19200:  S0BG   = 0x001F; break;   /* set baudrate to  19200 baud     */
   case  38400:  S0BG   = 0x000F; break;   /* set baudrate to  38400 baud     */
   case  57600:  S0BG   = 0x000A; break;   /* set baudrate to  57600 baud     */
   case 625000:  S0BG   = 0x0000; break;   /* set baudrate to 625000 baud     */
   }

  S0R = 1;                             /* start ASC0						  */

/*  end of init serial0 port:         								          */
}
