/*********************************************************************
  CAPCOMx_1.c          												 
  CAPCOMx-Unit (x = 1 or 2) in the capture mode (CCMODx = 0xx) for 
  measuring of signal frequency (period) with Capture/Compare register
  CC0. Signal input on pin P2.0. 
   
                            		 PHYTEC Messtechnik GmbH, 2000
 *********************************************************************/

/* -------Including Files: ----------------------------------------- */
#include <REG167.H>          /* Register defin for C167              */
#include <INTRINS.H>         /* Prototyping of specific Funktionen 
								for C167   							 */
#include <STDIO.H>           /* Standard I/O functions               */
#include "ASC0_lib.h"        /* init serial interface                */
#include "PWM.h"             /* init PWM - Unit                      */


#define CountEnd 65535
 
sbit DCC0IO = DP2^0;
unsigned char i;
unsigned int  mes[2], Offset;
unsigned long Period;

/* ----- Timer 0 Overflow Interrupt Program ------------------------ */
void Overflow (void) interrupt 0x20
{
  Offset++;
}

/* ----- Capture Interrupt Programm -------------------------------- */
void Capture (void) interrupt 0x10
{
/* the evaluation after measure of two edge			                 */
  mes[i] = CC0;                                                
                                                                 
   if (i == 1)		         /* after both measure			         */
   { 
     T0R  = 0;               /* stop timer T3                        */                                                                
     CC0IE = 0;		         /* diseble interrupt for capture        */ 
      if (Offset == 0)           /* if signal shorter is as timer cycle	 */
            Period = (long)(mes[1] - mes[0])*400;
         else                /* if signal slower is as timer cycle 	*/
            Period = (((long)CountEnd - mes[0]) +mes[1] + (Offset - 1)*(long)CountEnd)*400; 
     i = 0;
     T0R  = 1;               
     CC0IE = 1;	    	     /* enable interrupt for capture	    */	
   }
   else
   i++;
  Offset = 0;
}

/* ------ main Program: -------------------------------------------- */
void main (void)
{

  serial(57600);             /* init serial interface with 57600baud*/
  PWM();					 /* start PWM-Unit for test signal      */


  DCC0IO = 0;				 /* pin direction: input                 */
  
 /* ------------------------------------------------------------------------------------ */
 /* x = 0, 1, 7, 8,                Fcpu = 20 MHz                                         */
 /* ------------------------------------------------------------------------------------ */
 /* TxI | TxI |         Input          | Resolution | Overflow  |  Output   |   Output   */
 /* d/h | bin |         Clock          |            |  Period   | Frequency |   Period   */
 /* ------------------------------------------------------------------------------------ */
 /*  0  | 000 | Fcpu/8    =   2.50 MHz |    400 ns  |  26.21 ms | 19.073 Hz |  52.428 ms */
 /*  1  | 001 | Fcpu/16   =   1.25 MHz |    800 ns  |  52.43 ms |  9.536 Hz | 104.857 ms */
 /*  2  | 010 | Fcpu/32   = 625.00 kHz |   1600 ns  | 104.86 ms |  4.768 Hz | 209.715 ms */
 /*  3  | 011 | Fcpu/64   = 312.50 kHz |   3200 ns  | 209.71 ms |  2.284 Hz | 419.430 ms */
 /*  4  | 100 | Fcpu/128  = 156.25 kHz |   6400 ns  | 419.43 ms |  1.192 Hz | 838.861 ms */
 /*  5  | 101 | Fcpu/256  =  78.125kHz |  12800 ns  | 838.86 ms |  0.596 Hz |   1.677 s  */
 /*  6  | 110 | Fcpu/512  =  39.06 kHz |  25600 ns  |   1.677 s |  0.298 Hz |   3.355 s  */
 /*  7  | 111 | Fcpu/1024 =  19.53 kHz |  51200 ns  |   3.355 s |  0.149 Hz |   6.710 s  */
 /* ------------------------------------------------------------------------------------ */ 
 /*    Overflow Period  = 1/[Fcpu/(8*(2^TxI)*(2^16)]                                     */
 /*    Output Frequency = Fcpu/[8*(2^TxI)*(2^16)*2]                                      */
 /*    Output Period    = 1/{Fcpu/[8*(2^TxI)*(2^16)*2]}                                  */
 /* ------------------------------------------------------------------------------------ */


  T01CON = 0x0000 | 0;		 /* T0 as timer with Fcpu/8 input: 		 */
  T0IC   = 0x0058;           /* enable interrupt for overflow of timer0
								ILVL = 6; GLVL = 0; T0IE = 1.        */

  CCM0   = 0x0001;			 /* init CC0 register for capture with 
								rising edge, works with T0           */
  CC0IC  = 0x0054;           /* enable interrupt for CC0:
								ILVL = 5; GLVL = 0; CC0IE = 1.       */

  IEN   = 1;				 /* enable global interrupt              */

  T0R = 1;			 		 /* start Timer T3 and T4                */

  printf("\n");

   while (1)                 /* endless loop	                     */
   {
     /* here your program    		                                 */
       printf("\r Signal period = %10lu ns", Period);
   	 /* here your program    		                                 */
   } 
}

