/*********************************************************************
 CAPCOMx_3.c
                                                                   
   A/D-Converter in Single Channel Continuous Mode for Channel 0    
   with Channel Injection on Channel 2 started per Capture/Compare
   Register form CAPCOM2 Unit.

         							PHYTEC Metechnik GmbH 2000
 *********************************************************************/
/* ----- Include Dateien: ------------------------------------------ */
#include <REG167.H>          /* Register Definition for C167         */
#include <INTRINS.H>         /* Device specific functions            */
#include <STDIO.H>			 /* Standard I/O functions               */
#include "ASC0_lib.h"        /* serial interface function			 */
#include "LED.h"

/* ----- C-global Byte-/Word-Objects: ----------------------------- */
unsigned int ad_injected;   
unsigned int ad_value;
unsigned char command;

/* ----- Compare Interrupt Program for CC31 ----------------------- */
void Compare (void) interrupt 0x46
{
 LED = !LED;
}

/* ----- AD-Conversion ready interrupt program --------------------- */
void ad (void) interrupt 0x28
{
   ad_value = ADDAT & 0x3ff;	  /* read result of conversion without
								     channel number		             */
  printf("\r AD Value = %4u ",ad_value);
}

/* ----- AD-Conversion Error Interrupt program: -------------------- */
void ade_int ( void ) interrupt 0x29
{  
   ad_injected = ADDAT2 & 0x3ff; /* Result only, without Channel
									number	    				    */
   printf("AD Injected = %4u ",ad_injected);
}

/* ----- main program: -------------------------------------------- */
void main ( )
{  
   LED_on;
   serial(57600);            /* Init serial interface for 9600 baud  */
   putchar(' ');             /* Send dummy-Byte for compatibility    */
   printf("\n A-D Converter Channel Injection with CAPCOM2 \n");  
   
   /* ----- init A/D Converter ------------------------------------- */    
   /* The analog inputs is always define as inputs                   */
   ADCON  = 0x610;           /* Init A/D Converter for Single Channel
								Continuous Conversion on Channel 0,
								enable "Wait for Read and Channel
								Injection           */
   ADDAT2 = 0x2000;          /* Channel Injection on Channel 2       */
   ADEIC  = 0x63;            /* Init Channel-Injection-Interrupt     */
   ADCIC  = 0x62;            /* Init AD-Convesion ready interrupt    */
   ADST   = 1;               /* Start A/D-Conversion                 */

   /* ----- init Timer 7 and CC31 for ADC Channel Injection -------- */
   _bfld_(T78CON, 0x00FF, 0x0007);/* init T7 with Fcpu/1024          */
   T7REL = -58594;           /* reload value for period = 3 s        */
   _bfld_(CCM7, 0xF000, 0x4000);  /* init CC31 in compare mode 0
   									 works with T7                   */
   CC31 = -10;               /* reload value                         */
   CC31IC = 0x0064;          /* enable interrupt for CC31:
								ILVL = 9; GLVL = 0; CC31IE = 1.      */
   T7R = 1;                  /* start Timer 7                        */  

   IEN   = 1;                /* enable global interrupt              */


   while (1)                 /* Endless loop                         */
   {
      /* Here your program                                           */
   } 
}