/*********************************************************************
  GPT2_2.c          												 
   Timer T6 from General Purpose Timer Block 2 as Timer with variable
   input clock. BLINKY with delay time from Timer 6.
   
                            		 PHYTEC Messtechnik GmbH, 2000
 *********************************************************************/

/* -------Including Files: ----------------------------------------- */
#include <REG167.H>          /* Register defin for C167              */
#include <INTRINS.H>         /* Prototyping of specific Funktionen 
								for C167   							 */
#include "LED.h"
 
sbit DT6OUT = DP3^1;
unsigned char T6I;

/* ------ main Program: -------------------------------------------- */
void main (void)
{
  LED_on;  					 /* defined in LED.h                     */
  T6OUT  = 1;				 /* use pin as alternate function        */
  DT6OUT = 1;				 /* pin direction: output                */
  
             /* ----------------------------------------------------------------- */
             /*                     Fcpu = 20 MHz                                 */
             /* ----------------------------------------------------------------- */
             /* T6I |   Input   | Resolution | Overflow  |  Output   |   Output   */
	         /* bin |   Clock   |            |  Period   | Frequency |   Period   */
             /* ----------------------------------------------------------------- */
//  T6I = 0; /* 000 | Fcpu/4    |   200 ns   |  13.11 ms | 38.146 Hz |  26.214 ms */
//  T6I = 1; /* 001 | Fcpu/8    |   400 ns   |  26.21 ms | 19.073 Hz |  52.428 ms */ 
//  T6I = 2; /* 010 | Fcpu/16   |   800 ns   |  52.43 ms |  9.536 Hz | 104.857 ms */
    T6I = 3; /* 011 | Fcpu/32   |   1.6 s   | 104.56 ms |  4.768 Hz | 209.715 ms */
//  T6I = 4; /* 100 | Fcpu/64   |   3.2 s   | 209.71 ms |  2.284 Hz | 419.430 ms */
//  T6I = 5; /* 101 | Fcpu/128  |   6.4 s   | 419.43 ms |  1.192 Hz | 838.861 ms */
//  T6I = 6; /* 110 | Fcpu/256  |   12.8 s  |  828.86 s |  0.596 Hz |   1.677 s  */
//  T6I = 7; /* 111 | Fcpu/512  |  28.6 s   |   1.677 s |  0.298 Hz |   3.355 s  */
             /* ----------------------------------------------------------------- */ 
             /*    Overflow Period  = 1/[Fcpu/(4*(2^T6I)*(2^16)]                  */
			 /*    Output Frequency = Fcpu/[4*(2^T6I)*(2^16)*2]                   */
			 /*    Output Period    = 1/{Fcpu/[4*(2^T6I)*(2^16)*2]}               */
			 /* ----------------------------------------------------------------- */

  T6CON = 0x0200 | T6I;		 /* T6 as timer with Fcpu/32 input: 
								=> interrupt request all 104.56 ms
								becaus of Otput Toggle Latch on pin
								P3.1 frequency = 4.768 Hz   		 */

  T6R   = 1;                 /* start Timer T6                       */


   while (1)                 /* endless loop	                     */
   {
     /* here your program    		                                 */

     LED = T6OTL;
     /* here your program    		                                 */
   } 
}

