/*********************************************************************
  GPT2_3.c          												 
   Timer T6 from General Purpose Timer Block 2 as Timer in gated mode
   for measuring of high-impuls width with input clock Fcpu/4 = 200 ns.
   Timer T5 initialize as a counter with imput from T6OTL and with 
   timer T6 work that as a 32-bit timer for measuring of long impulses
   (width < 4 sek.).
   
                            		 PHYTEC Messtechnik GmbH, 2000
 *********************************************************************/

/* -------Including Files: ----------------------------------------- */
#include <REG167.H>          /* Register defin for C167              */
#include <INTRINS.H>         /* Prototyping of specific Funktionen 
								for C167   							 */
#include <STDIO.H>           /* Standard I/O functions               */
#include "ASC0_lib.h"        /* init serial interface                */
#include "PWM.h"             /* init PWM - Unit                      */
 

unsigned char T6I;
unsigned long Width;

/* ------ main Program: -------------------------------------------- */
void main (void)
{

  serial(57600);             /* init serial interface with 57600baud*/
  PWM();					 /* start PWM-Unit for test signals      */

/* The Port 5 is always input */
  
             /* ----------------------------------------------------------------- */
             /*                     Fcpu = 20 MHz                                 */
             /* ----------------------------------------------------------------- */
             /* T6I |   Input   | Resolution | Overflow  |  Output   |   Output   */
	         /* bin |   Clock   |            |  Period   | Frequency |   Period   */
             /* ----------------------------------------------------------------- */
    T6I = 0; /* 000 | Fcpu/4    |   200 ns   |  13.11 ms | 38.146 Hz |  26.214 ms */
//  T6I = 1; /* 001 | Fcpu/8    |   400 ns   |  26.21 ms | 19.073 Hz |  52.428 ms */ 
//  T6I = 2; /* 010 | Fcpu/16   |   800 ns   |  52.43 ms |  9.536 Hz | 104.857 ms */
//  T6I = 3; /* 011 | Fcpu/32   |   1.6 s   | 104.56 ms |  4.768 Hz | 209.715 ms */
//  T6I = 4; /* 100 | Fcpu/64   |   3.2 s   | 209.71 ms |  2.284 Hz | 419.430 ms */
//  T6I = 5; /* 101 | Fcpu/128  |   6.4 s   | 419.43 ms |  1.192 Hz | 838.861 ms */
//  T6I = 6; /* 110 | Fcpu/256  |   12.8 s  |  828.86 s |  0.596 Hz |   1.677 s  */
//  T6I = 7; /* 111 | Fcpu/512  |  28.6 s   |   1.677 s |  0.298 Hz |   3.355 s  */
             /* ----------------------------------------------------------------- */ 
             /*    Overflow Period  = 1/[Fcpu/(4*(2^T6I)*(2^16)]                  */
			 /*    Output Frequency = Fcpu/[4*(2^T6I)*(2^16)*2]                   */
			 /*    Output Period    = 1/{Fcpu/[4*(2^T6I)*(2^16)*2]}               */
			 /* ----------------------------------------------------------------- */

  T6CON = 0x0018 | T6I;		 /* T6 as timer with Fcpu/4 = 200 ns input   
								in gated mode (High active)          */

  T5CON = 0x000F;            /* T5 as counter with input clock from
                                T6OTL. (T6 + T5 as 32-bit Timer )    */
   /* or  */
/* T5CON = 0x000D            /* T5 as counter with input clock from
        or 0x000E               T6OTL. (T6 + T5 as 33-bit Timer )    */

  T6 = T5 = 0;				 /* start value for Timer T6 and T5      */

  printf("\n");

   while (1)                 /* endless loop	                     */
   {
     /* here your program    		                                 */
     if(T6IN)			 
     {
       while(T6IN);          /* wait as long as 'T6IN' is High	     */
       T6R = T6R = 0;        /* stop Timer T3 and T4                 */
       Width =((long)(65535)* T5 +(long)(T6))*200;
       printf("\r Width of High Impulse = %10lu ns,( T6 = %5x hex., T5 = %5x hex.) ", Width,T6,T5);
       T6 = T5 = 0;			 /* clear Timer T6 and T5                */
       while(T6IN);          /* wait as long as 'T6IN' is High	     */
       T6R = T5R = 1;        /* stert Timer T6 and T5                */
     }
     
	 /* here your program    		                                 */
   } 
}

