/*********************************************************************
  GPT2_7.c          												 
   Timer T5 from General Purpose Timer Block 2 as Timer with input 
   clock Fcpu/4 = 200 ns for measure of frequency, start value T5 = 0,
   count directions: UP. Through rising edge on P3.2/CAPIN is T5 in 
   the CAPREL saved and with 0000H loaded. The interrupt program at 
   overflow of timer 5 counts the overflows.
   The measuring periode of signal is as
     Period = (CAPREL + 2^16*Overflows(Offsets))*200 calculated.

   
                            		 PHYTEC Messtechnik GmbH, 2000
 *********************************************************************/

/* -------Including Files: ----------------------------------------- */
#include <REG167.H>          /* Register defin for C167              */
#include <INTRINS.H>         /* Prototyping of specific Funktionen 
								for C167   							 */
#include <STDIO.H>           /* Standard I/O functions               */
#include "ASC0_lib.h"        /* init serial interface                */
#include "PWM.h"             /* init PWM - Unit                      */


#define CountEnd 65535

sbit DCAPIN = P3^2;
 
unsigned char T5I;
unsigned int  Offset = 0;
unsigned long Period;

/* ----- Overflow Inrerrupt Programm ------------------------------- */
void Overflow (void) interrupt 0x25
{
  Offset++;
}

/* ----- Capture Interrupt Programm -------------------------------- */
void Capture (void) interrupt 0x27
{
   Period = (long)(CAPREL + Offset * CountEnd)*200;
   Offset = 0;               /* clear Offset                         */
}


/* ------ main Program: -------------------------------------------- */
void main (void)
{

  serial(57600);             /* init serial interface with 57600baud*/
  PWM();					 /* start PWM-Unit for test signal      */

  DCAPIN = 0;                /* direction for pin as input          */ 

  
             /* ----------------------------------------------------------------- */
             /*                     Fcpu = 20 MHz                                 */
             /* ----------------------------------------------------------------- */
             /* T5I |   Input   | Resolution | Overflow  |  Output   |   Output   */
	         /* bin |   Clock   |            |  Period   | Frequency |   Period   */
             /* ----------------------------------------------------------------- */
    T5I = 0; /* 000 | Fcpu/4    |   200 ns   |  13.11 ms | 38.146 Hz |  26.214 ms */
//  T5I = 1; /* 001 | Fcpu/8    |   400 ns   |  26.21 ms | 19.073 Hz |  52.428 ms */ 
//  T5I = 2; /* 010 | Fcpu/16   |   800 ns   |  52.43 ms |  9.536 Hz | 104.857 ms */
//  T5I = 3; /* 011 | Fcpu/32   |   1.6 s   | 104.56 ms |  4.768 Hz | 209.715 ms */
//  T5I = 4; /* 100 | Fcpu/64   |   3.2 s   | 209.71 ms |  2.284 Hz | 419.430 ms */
//  T5I = 5; /* 101 | Fcpu/128  |   6.4 s   | 419.43 ms |  1.192 Hz | 838.861 ms */
//  T5I = 6; /* 110 | Fcpu/256  |   12.8 s  |  828.86 s |  0.596 Hz |   1.677 s  */
//  T5I = 7; /* 111 | Fcpu/512  |  28.6 s   |   1.677 s |  0.298 Hz |   3.355 s  */
             /* ----------------------------------------------------------------- */ 
             /*    Overflow Period  = 1/[Fcpu/(4*(2^T5I)*(2^16)]                  */
			 /*    Output Frequency = Fcpu/[4*(2^T5I)*(2^16)*2]                   */
			 /*    Output Period    = 1/{Fcpu/[4*(2^T5I)*(2^16)*2]}               */
			 /* ----------------------------------------------------------------- */

  T5CON = 0xD000 | T5I;		 /* T5 as timer with Fcpu/4 = 200 ns input
                                enable capture for CAPREL, rising edge*/
  T5IC  = 0x0060;			 /* enable interrupt for overflow of T5
                                ILVL = 8; GLVL = 0; T5IE = 1;        */
  CRIC  = 0x0050;            /* enable interrupt for caprute register
                                CAPREL: ILVL = 7; GLVL = 0; CRIE = 1;*/
  IEN   = 1;				 /* enable global interrupt              */
  T5R = 1;			         /* start Timer T5                       */

  printf("\n");

   while (1)                 /* endless loop	                     */
   {
     /* here your program    		                                 */
       printf("\r Signal period = %10lu ns ", Period);
   	 /* here your program    		                                 */
   } 
}

