/*********************************************************************
  PWM4.c for PHYTEC mM167          												 
                                                                 
  The PWM-Unit generate a PWM-Signal on PWM-Channel 1 Edge Aligned
   PWM with constant frequecy (period) and variable puls width 
   dependent on AD-Converter value an AD-Channel 1 in a interrupt 
   service routine.
  The PWM-Timer 1 work with CPU-Clock (20 MHz)                     
  The period register contain the value 40000:                     
      Period a PWM-Signal = 20 MHz /(40000-1) = 250 Hz    
      Output at Port pin P7.1/POUT1 High-aktive (pin B42).
  The PWM-Interrupt PWMIR load puls width register with a PWM-value
      from AD-Converter an AD-Channel 1.                     
       ILVL = 13, GLVL = 3, PWMIE = 1, IEN = 1          

									 PHYTEC Messtechnik GmbH, 1999
**********************************************************************/
#pragma mod167
/* -------Including Files: ----------------------------------------- */
#include <REG167.H>          /* defin Register for C167              */
sbit DP71 = DP7^1;           /* Bitname DP71 for DP7.1 define        */
#include <INTRINS.H>         /* Prototyping of specific Funktionen 
								for C176   							 */

unsigned int PW1_new;

/* --------------------------AD-Converter----------------------------*/
unsigned int ad_conv (unsigned char ADC_Channel)
{
unsigned int Result;
/* init AD-Converter												 */
 ADCON = 0x00 + ADC_Channel;  /* A/D-Converter: Channel0, Single     */	
				       		  /* Channel Conversion		      	     */
   ADST = 1;			      /* Conversion start		      		 */
     while(ADBSY);		      /* wait of end of conversion	      	 */
   Result = ADDAT & 0x03FF;	  /*read result of conversion 			 */
 return Result;
}

/* ------ Code: PWM-Interruptprogramm: ----------------------------- */
void pwm_int ( void ) interrupt 0x3f
{
   switch (PWMCON0 & 0xf000) /* query about a PWM-Channel number     */
   {  case 0x2000: PIR1 = 0; /* Channel 1-Interrupt: request bit
                                clear and                            */
     PW1 = PW1_new;          /* puls width register new load         */
      break;
     default: PWMCON0 = PWMCON0 & 0x0fff; /* else clear all request 
	 										 bits                	 */
   }
}  


/* ------ main Program: -------------------------------------------  */
void main (void)
{ 
   DP71  = 1;                /* Define PWM-Output as Push-/Pull-
                                Output, Latch-contents after Reset 0 */
   PM1   = 0;                /* PWM-Channel 1 Edge Aligned Mode    */
   PP1   = 40000;	         /* Load period register                 */
   PW1   = 20000;            /* Load puls width register             */
   PEN1  = 1;                /* P7.1/POUT1 release                   */
   PWMIC = 0x77;             /* configure PWM-Interrupt              */
   PIE1  = 1;                /* enable individual PWM1-Interrupt     */
   IEN   = 1;				 /* Interrupt Request enable			 */
   PTR1  = 1;                /* Start of PWM-Timer 1                 */

   while (1)                 /* endless loop	                     */
   {
     PW1_new = (ad_conv(0)<<6); /* load Puls withe register with new 
								value from 0 - 100%					 */
   } 
}
/* -- End of program ----------------------------------------------- */
