/****************************************************************/
/* FILENAME:    UART.C                                          */
/* VERSION:     1.1                                             */
/* DATE:        20.01.95                                        */
/* AUTOR:       D. Heger                                        */
/* REVISED:     26.04.95, B.Huber                               */
/*                                                              */
/* REV 1.2 Baudratenfehler mit 38400bps entfernt,24.02.2000 M.S.*/
/* (c) 1994 PHYTEC Messtechnik GmbH Mainz                       */
/****************************************************************/
#pragma large                       /* ggf. Speichemodell anpassen */

#pragma MOD167 FIX167

#include <reg167.h>
#include <stdio.h>
#include <absacc.h>

#ifndef word
 #define word unsigned int                   /* Typen-Abkrzung */
#endif
#ifndef byte
 #define byte unsigned char
#endif

#define UART_BASE_ADDRESS 0x400000          /* Absolute Adr. des UART */

 /* Relative Adr. d. UART (Register)*/
 #define MR1 0x0 + UART_BASE_ADDRESS	/* MODE Reg.1 R/W */
 #define MR2 0x0 + UART_BASE_ADDRESS	/* MODE Reg.2 R/W */
 #define SR  0x1 + UART_BASE_ADDRESS	/* SatusReg.  R   */
 #define CSR 0x1 + UART_BASE_ADDRESS	/* ClockselReg. W */
 #define CR  0x2 + UART_BASE_ADDRESS	/* CommandReg.	W */
 #define RHR 0x3 + UART_BASE_ADDRESS	/* RxD Reg. 	R */
 #define THR 0x3 + UART_BASE_ADDRESS	/* TxD Reg. 	W */
 #define ACR 0x4 + UART_BASE_ADDRESS	/* TxD Reg. 	W */

/****************************************************************************/
/* FUNCTION rd_UART                                                         */
/*                                                                          */
/* benutzte Funktionen  : keine                                             */
/* bergabeparameter    : Register Adresse (0..7)                           */
/* Rckgabewert         : D-Byte                                            */
/****************************************************************************/
byte rd_UART (byte adr)     /* read byte from UART adr. */
{
return(MVAR(byte, UART_BASE_ADDRESS + adr));
}
                  
/****************************************************************************/
/* FUNCTION wr_UART                                                         */
/*                                                                          */
/* benutzte Funktionen  : keine                                             */
/* bergabeparameter    : Register Adresse (0..7), D-Byte                   */
/* Rckgabewert         : keine                                             */
/****************************************************************************/
void wr_UART (byte adr, byte dat)       /* write byte to UART adr. */
{
MVAR(byte, UART_BASE_ADDRESS + adr) = dat;
}

/****************************************************************************/
/* FUNCTION init_UART                                                       */
/*                                                                          */
/* benutzte Funktionen  : MVAR                                              */
/* bergabeparameter    : INITTYPE -Byte                                    */
/* Rckgabewert         : -                                                 */
/* init uart, clear error flags, enable receiver and transmiter
   in one of this modes:
   bit7..5 = baudrate + bit4..3 = data  + bit2..1 = Parity + bit0 = Stopbit
   00h 000 =   300bps + 00h  00 = 5 BIT + 00h  00 = Even   + 00h 0 = 1
   20h 001 =   600bps + 08h  01 = 6 BIT + 02h  01 = Odd    + 01h 1 = 2
   40h 010 =  1200bps + 10h  10 = 7 BIT + 04h  10 = Mark
   60h 011 =  2400bps + 18h  11 = 8 BIT + 06h  11 = No
   80h 100 =  4800bpd
   A0h 101 =  9600bps
   C0h 110 = 19200bps
   E0h 111 = 38400bps                                                       */
/****************************************************************************/
void init_UART (byte inittype)
{
byte baudrate, parity, dbits, sbits;
byte mr1, mr2, csr, acr;

acr = 0x08;                             /* BRG set 0 (38.4kbps), UART on */

mr1 = inittype >> 5;                    /* nur baurate bits normiert*/
baudrate = mr1 + 4;
if  (mr1 > 2)   baudrate += 1;
if  (mr1 > 4)   baudrate += 1;
if  (mr1 == 7)  baudrate = 0x0C;        /* NEW M.Schneider for set 38.4kbs */ 

csr = (baudrate<<4) | baudrate;         /* Rx Clock = Tx Clock = CSR */

if      (mr1 == 6)  acr = 0x88;         /* BRG set 1 (19.2kbps), UART on */

parity = inittype & 0x06;               /* nur parity bits */
parity = parity << 1;
if (parity == 0x0C)   parity = 0x10;    /* alle parity bits ok */

dbits = (inittype & 0x18) >> 3;         /* nur dbits normiert */

mr1 = parity | dbits;                   /* init wert fr MR1 */

sbits = inittype & 0x01;                /* nur sbits normiert */

if (sbits == 0)     mr2 = 0x07;         /* init wert fr MR2 */
else                mr2 = 0x0F;


MVAR(char, ACR) = acr;  /* UART aktivieren */
MVAR(char, CR) = 0x1A;  /* setze MR-Pointer auf MR1, Disable TxD und RxD */
MVAR(char, MR1) = mr1;  /* MR1 init zeiger geht auf MR2 */
MVAR(char, MR2) = mr2;  /* MR2 init */
MVAR(char, CSR) = csr;  /* Baudrate einstellen */
MVAR(char, CR) = 0x45;  /* reset RxD Buf., Enable TxD und RxD */

//printf ("\nCSR-Ausgabe %02X",csr);     /* the 'printf' function call               */
//printf ("\nBaudrate-Ausgabe %02X",baudrate);  /* the 'printf' function call               */


return;
}

/****************************************************************************/
/* FUNCTION getstat_UART                                                    */
/*                                                                          */
/* benutzte Funktionen  : keine                                             */
/* bergabeparameter    : keine                                             */
/* Rckgabewert         : Statusbyte                                        */
/****************************************************************************/
byte getstat_UART (void)   /* read statusregister byte from receiver */
{
byte status;
status = MVAR(char, SR);
return(status);
}

/****************************************************************************/
/* FUNCTION wrtx_UART                                                       */
/* schreibt ein byte nach TX_buf ohne status kontrolle                      */
/*                                                                          */
/* benutzte Funktionen  : keine                                             */
/* bergabeparameter    : Datenbyte zum senden                              */
/* Rckgabewert         : -                                                 */
/****************************************************************************/
void wrtx_UART (byte dat)    /* write byte to transmitter */
{
 MVAR(char, THR) = dat;
 return;
}

/****************************************************************************/
/* FUNCTION putc_UART                                                       */
/* sendet ein byte ber TX_buf mit status kontrolle                         */
/*                                                                          */
/* benutzte Funktionen  : keine                                             */
/* bergabeparameter    : Datenbyte zum senden                              */
/* Rckgabewert         : -                                                 */
/****************************************************************************/
void putc_UART (byte dat)    /* write byte to transmitter */
{
 byte stat = 0;
 while (stat == 0) {
    stat = getstat_UART();
    stat &= 0x04;
    }
 MVAR(char, THR) = dat;
 return;
}

/****************************************************************************/
/* FUNCTION rdrx_UART                                                       */
/* liest ein byte von RX_buf ohne staus kontrolle                           */
/*                                                                          */
/* benutzte Funktionen  : keine                                             */
/* bergabeparameter    : keine                                             */
/* Rckgabewert         : Byte aus RxD                                      */
/****************************************************************************/
byte rdrx_UART (void)   /* read byte from receiver */
{
byte dat;
dat = MVAR(char, THR);
return(dat);
}

/****************************************************************************/
/* FUNCTION getc_UART                                                       */
/* empfngt ein byte von RX_buf mit status kontrolle                        */
/*                                                                          */
/* benutzte Funktionen  : keine                                             */
/* bergabeparameter    : keine                                             */
/* Rckgabewert         : Byte aus RxD                                      */
/****************************************************************************/
byte getc_UART (void)   /* read byte from receiver */
{
byte dat = 0;
 while (dat == 0) {
    dat = getstat_UART();
    dat &= 0x01;
    }
dat = MVAR(char, THR);
return(dat);
}

